import torch
import torch.nn as nn
import json
import os 


class AIIAConfig:
    def __init__(
        self,
        model_name: str = "AIIA",
        kernel_size: int = 5,
        activation_function: str = "GELU",
        hidden_size: int = 512,
        num_hidden_layers: int = 12,
        num_channels: int = 3,
        learning_rate: float = 5e-5,
        **kwargs
    ):
        self.model_name = model_name
        self.kernel_size = kernel_size
        self.activation_function = activation_function
        self.hidden_size = hidden_size
        self.num_hidden_layers = num_hidden_layers
        self.num_channels = num_channels
        self.learning_rate = learning_rate
        
        # Store additional keyword arguments as attributes
        for key, value in kwargs.items():
            setattr(self, key, value)

    @property
    def activation_function(self):
        return self._activation_function

    @activation_function.setter
    def activation_function(self, value):
        attr = getattr(nn, value, None)
        if attr is None or (not callable(attr) and not isinstance(attr, type(nn.Module))):
            valid_funcs = [func for func in dir(nn) if callable(getattr(nn, func)) or isinstance(getattr(nn, func), type(nn.Module))]
            raise ValueError(f"Invalid activation function: {value}. Choose from: {', '.join(valid_funcs)}")
        self._activation_function = value

    def save(self, file_path):
        if not os.path.exists(file_path):
            os.makedirs(file_path, exist_ok=True)
        with open(f"{file_path}/config.json", 'w') as f:
            json.dump(vars(self), f, indent=4)

    @classmethod
    def load(cls, file_path):
        with open(f"{file_path}/config.json", 'r') as f:
            config_dict = json.load(f)
        return cls(**config_dict)